import { VirtualElement, AutoUpdateOptions as AutoUpdateOptions$1 } from '@floating-ui/dom';
export { VirtualElement } from '@floating-ui/dom';
import * as _visoning_vue_utility from '@visoning/vue-utility';
import { MaybeRef } from '@visoning/vue-utility';
import { Strategy, Placement, MiddlewareData, ComputePositionConfig, Middleware } from '@floating-ui/core';
export { Middleware, MiddlewareData, Placement, Strategy } from '@floating-ui/core';
import * as vue_demi from 'vue-demi';
import { Ref, PropType, UnwrapRef, ExtractPropTypes } from 'vue-demi';
import * as _floating_ui_core_src_types from '@floating-ui/core/src/types';

declare type ReferenceType = Element | VirtualElement;
declare type FloatingType = HTMLElement;
declare type MaybeRefType<T> = MaybeRef<T | undefined | null>;
declare type MaybeReferenceRef = MaybeRefType<ReferenceType>;
declare type MaybeFloatingRef = MaybeRefType<FloatingType>;
interface FloatingData {
    x: number | null;
    y: number | null;
    strategy: Strategy;
    placement: Placement;
    middlewareData: MiddlewareData;
}

declare type UseAutoUpdateOptions = Partial<AutoUpdateOptions$1> & {
    /**
     * @default false
     */
    disabled?: boolean;
};
/**
 * Automatically updates the position of the floating element when necessary.
 * @see https://floating-ui.com/docs/autoUpdate
 */
declare function useAutoUpdate(reference: MaybeReferenceRef, floating: MaybeFloatingRef, update: () => void, options?: MaybeRef<UseAutoUpdateOptions>): () => void;

declare type FloatingOptions = Omit<ComputePositionConfig, 'platform'> & {
    disabled?: boolean;
};

declare type UseFloatingOptions = FloatingOptions;
interface FloatingControl {
    /**
     * @see https://floating-ui.com/docs/computePosition#return-value
     */
    data: Readonly<Ref<FloatingData>>;
    update: () => void;
    stop: () => void;
}
/**
 * Uue hooks based on `computePosition`.
 * @see https://floating-ui.com/docs/computePosition
 */
declare function useFloating(referenceRef: MaybeReferenceRef, floatingRef: MaybeFloatingRef, options?: MaybeRef<UseFloatingOptions>): FloatingControl;

declare const FloatingCreator: vue_demi.DefineComponent<_visoning_vue_utility.WithDefaultProps<{
    readonly reference: vue_demi.PropType<ReferenceType | null>;
    readonly floating: vue_demi.PropType<HTMLElement | null>;
    readonly disabled: BooleanConstructor;
    readonly placement: vue_demi.PropType<_floating_ui_core_src_types.Placement>;
    readonly strategy: vue_demi.PropType<_floating_ui_core_src_types.Strategy>;
    readonly middleware: vue_demi.PropType<_floating_ui_core_src_types.Middleware[]>;
    readonly autoUpdate: vue_demi.PropType<boolean | UseAutoUpdateOptions>;
    readonly onFloatingDataUpdate: vue_demi.PropType<(data: {
        x: number | null;
        y: number | null;
        strategy: _floating_ui_core_src_types.Strategy;
        placement: _floating_ui_core_src_types.Placement;
        middlewareData: {
            [x: string]: any;
            arrow?: {
                x?: number | undefined;
                y?: number | undefined;
                centerOffset: number;
            } | undefined;
            autoPlacement?: {
                index?: number | undefined;
                overflows: {
                    placement: _floating_ui_core_src_types.Placement;
                    overflows: number[];
                }[];
            } | undefined;
            flip?: {
                index?: number | undefined;
                overflows: {
                    placement: _floating_ui_core_src_types.Placement;
                    overflows: number[];
                }[];
            } | undefined;
            hide?: {
                referenceHidden?: boolean | undefined;
                escaped?: boolean | undefined;
                referenceHiddenOffsets?: {
                    top: number;
                    right: number;
                    bottom: number;
                    left: number;
                } | undefined;
                escapedOffsets?: {
                    top: number;
                    right: number;
                    bottom: number;
                    left: number;
                } | undefined;
            } | undefined;
            offset?: {
                x: number;
                y: number;
            } | undefined;
            shift?: {
                x: number;
                y: number;
            } | undefined;
        };
    }) => void>;
}, {
    readonly placement: "bottom";
    readonly strategy: "absolute";
    readonly autoUpdate: true;
}>, () => vue_demi.VNode<vue_demi.RendererNode, vue_demi.RendererElement, {
    [key: string]: any;
}>[] | undefined, unknown, {}, {}, vue_demi.ComponentOptionsMixin, vue_demi.ComponentOptionsMixin, {}, string, vue_demi.VNodeProps & vue_demi.AllowedComponentProps & vue_demi.ComponentCustomProps, Readonly<vue_demi.ExtractPropTypes<_visoning_vue_utility.WithDefaultProps<{
    readonly reference: vue_demi.PropType<ReferenceType | null>;
    readonly floating: vue_demi.PropType<HTMLElement | null>;
    readonly disabled: BooleanConstructor;
    readonly placement: vue_demi.PropType<_floating_ui_core_src_types.Placement>;
    readonly strategy: vue_demi.PropType<_floating_ui_core_src_types.Strategy>;
    readonly middleware: vue_demi.PropType<_floating_ui_core_src_types.Middleware[]>;
    readonly autoUpdate: vue_demi.PropType<boolean | UseAutoUpdateOptions>;
    readonly onFloatingDataUpdate: vue_demi.PropType<(data: {
        x: number | null;
        y: number | null;
        strategy: _floating_ui_core_src_types.Strategy;
        placement: _floating_ui_core_src_types.Placement;
        middlewareData: {
            [x: string]: any;
            arrow?: {
                x?: number | undefined;
                y?: number | undefined;
                centerOffset: number;
            } | undefined;
            autoPlacement?: {
                index?: number | undefined;
                overflows: {
                    placement: _floating_ui_core_src_types.Placement;
                    overflows: number[];
                }[];
            } | undefined;
            flip?: {
                index?: number | undefined;
                overflows: {
                    placement: _floating_ui_core_src_types.Placement;
                    overflows: number[];
                }[];
            } | undefined;
            hide?: {
                referenceHidden?: boolean | undefined;
                escaped?: boolean | undefined;
                referenceHiddenOffsets?: {
                    top: number;
                    right: number;
                    bottom: number;
                    left: number;
                } | undefined;
                escapedOffsets?: {
                    top: number;
                    right: number;
                    bottom: number;
                    left: number;
                } | undefined;
            } | undefined;
            offset?: {
                x: number;
                y: number;
            } | undefined;
            shift?: {
                x: number;
                y: number;
            } | undefined;
        };
    }) => void>;
}, {
    readonly placement: "bottom";
    readonly strategy: "absolute";
    readonly autoUpdate: true;
}>>>, {
    readonly placement: _floating_ui_core_src_types.Placement;
    readonly strategy: _floating_ui_core_src_types.Strategy;
    readonly disabled: boolean;
    readonly autoUpdate: boolean | UseAutoUpdateOptions;
}>;

declare type AutoUpdateOptions = UseAutoUpdateOptions;
declare type FloatingCreatorSlotProps = FloatingData;
interface FloatingCreatorExposed {
    updatePosition: () => void;
    getFloatingData: () => FloatingData;
}
declare const FloatingCreatorPropsType: {
    /**
     * Reference element.
     *
     * You can position a floating element relative to a virtual element instead of a real one.
     * This enables things like positioning context menus or following the cursor.
     * @see https://floating-ui.com/docs/virtual-elements
     */
    readonly reference: PropType<ReferenceType | null>;
    /**
     * Floating element.
     */
    readonly floating: PropType<HTMLElement | null>;
    /**
     * Whether to disable the floating.
     */
    readonly disabled: BooleanConstructor;
    /**
     * Where to place the floating element relative to its reference element.
     * @see https://floating-ui.com/docs/computePosition#placement
     *
     * @default 'bottom'
     */
    readonly placement: PropType<Placement>;
    /**
     * This is the type of CSS position property to use.
     * @see https://floating-ui.com/docs/computePosition#strategy
     *
     * @default 'absolute'
     */
    readonly strategy: PropType<Strategy>;
    /**
     * They alter the positioning coordinates from the basic placement to customize the position,
     * provide data, or act as "visibility optimization" techniques.
     * @see https://floating-ui.com/docs/computePosition#middleware
     */
    readonly middleware: PropType<Middleware[]>;
    /**
     * This function adds listeners that automatically call an update function when necessary
     * so that the floating element remains “anchored” to the reference element in a variety of scenarios without detaching.
     *
     * @default true
     */
    readonly autoUpdate: PropType<boolean | UseAutoUpdateOptions>;
    /**
     * Callback on floating data status changes.
     */
    readonly onFloatingDataUpdate: PropType<(data: UnwrapRef<FloatingData>) => void>;
};
declare const FloatingCreatorDefaultProps: {
    readonly placement: "bottom";
    readonly strategy: "absolute";
    readonly autoUpdate: true;
};
declare const FloatingCreatorProps: _visoning_vue_utility.WithDefaultProps<{
    /**
     * Reference element.
     *
     * You can position a floating element relative to a virtual element instead of a real one.
     * This enables things like positioning context menus or following the cursor.
     * @see https://floating-ui.com/docs/virtual-elements
     */
    readonly reference: PropType<ReferenceType | null>;
    /**
     * Floating element.
     */
    readonly floating: PropType<HTMLElement | null>;
    /**
     * Whether to disable the floating.
     */
    readonly disabled: BooleanConstructor;
    /**
     * Where to place the floating element relative to its reference element.
     * @see https://floating-ui.com/docs/computePosition#placement
     *
     * @default 'bottom'
     */
    readonly placement: PropType<Placement>;
    /**
     * This is the type of CSS position property to use.
     * @see https://floating-ui.com/docs/computePosition#strategy
     *
     * @default 'absolute'
     */
    readonly strategy: PropType<Strategy>;
    /**
     * They alter the positioning coordinates from the basic placement to customize the position,
     * provide data, or act as "visibility optimization" techniques.
     * @see https://floating-ui.com/docs/computePosition#middleware
     */
    readonly middleware: PropType<Middleware[]>;
    /**
     * This function adds listeners that automatically call an update function when necessary
     * so that the floating element remains “anchored” to the reference element in a variety of scenarios without detaching.
     *
     * @default true
     */
    readonly autoUpdate: PropType<boolean | UseAutoUpdateOptions>;
    /**
     * Callback on floating data status changes.
     */
    readonly onFloatingDataUpdate: PropType<(data: UnwrapRef<FloatingData>) => void>;
}, {
    readonly placement: "bottom";
    readonly strategy: "absolute";
    readonly autoUpdate: true;
}>;
declare type FloatingCreatorProps = ExtractPropTypes<typeof FloatingCreatorProps>;

export { AutoUpdateOptions, FloatingControl, FloatingCreator, FloatingCreatorDefaultProps, FloatingCreatorExposed, FloatingCreatorProps, FloatingCreatorPropsType, FloatingCreatorSlotProps, FloatingData, FloatingType, MaybeFloatingRef, MaybeReferenceRef, ReferenceType, UseAutoUpdateOptions, UseFloatingOptions, useAutoUpdate, useFloating };
